enum AlternativeExprs<'db> {
Few(FxHashSet<Expr<'db>>),
Many,
}Expand description
Helper enum to squash big number of alternative trees into Many variant as there is too many
to take into account.
Variants
Few(FxHashSet<Expr<'db>>)
There are few trees, so we keep track of them all
Many
There are too many trees to keep track of
Implementations
Sourceimpl<'db> AlternativeExprs<'db>
impl<'db> AlternativeExprs<'db>
Sourcefn new(
threshold: usize,
exprs: impl Iterator<Item = Expr<'db>>,
) -> AlternativeExprs<'db>
fn new( threshold: usize, exprs: impl Iterator<Item = Expr<'db>>, ) -> AlternativeExprs<'db>
Construct alternative trees
Arguments
threshold - threshold value for many trees (more than that is many)
exprs - expressions iterator
Sourcefn exprs(&self, ty: &Type<'db>) -> Vec<Expr<'db>>
fn exprs(&self, ty: &Type<'db>) -> Vec<Expr<'db>>
Get type trees stored in alternative trees (or Expr::Many in case of many)
Arguments
ty - Type of expressions queried (this is used to give type to Expr::Many)
Sourcefn extend_with_threshold(
&mut self,
threshold: usize,
exprs: impl Iterator<Item = Expr<'db>>,
)
fn extend_with_threshold( &mut self, threshold: usize, exprs: impl Iterator<Item = Expr<'db>>, )
Extend alternative expressions
Arguments
threshold - threshold value for many trees (more than that is many)
exprs - expressions iterator
fn is_many(&self) -> bool